/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __POINT_H
#define __POINT_H

#include <math.h>

template<class T> class Point {
public:
	T x, y;
	Point() {
	}
	Point(T a, T b) {
		x = a;
		y = b;
	}
	Point operator+(const Point &p) const {
		Point out;
		out.x = x + p.x;
		out.y = y + p.y;
		return out;
	}
	Point operator-(const Point &p) const {
		Point out;
		out.x = x - p.x;
		out.y = y - p.y;
		return out;
	}
	Point operator-() const {
		Point out;
		out.x = -x;
		out.y = -y;
		return out;
	}
	Point operator/(T f) const {
		Point out;
		out.x = x/f;
		out.y = y/f;
		return out;
	}
	Point operator*(T f) const {
		Point out;
		out.x = x*f;
		out.y = y*f;
		return out;
	}
	float abs() const {
		return hypot(x,y);
	}
	static T dot(const Point &p1, const Point &p2) {
		return p1.x*p2.x + p1.y*p2.y;
	}
};

typedef Point<float> FPoint;
typedef Point<int> IPoint;

#endif
